# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    SimCardDeleteResponse,
    SimCardUpdateResponse,
    SimCardRetrieveResponse,
    SimCardGetPublicIPResponse,
    SimCardGetDeviceDetailsResponse,
    SimCardGetActivationCodeResponse,
    SimCardListWirelessConnectivityLogsResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.shared import SimpleSimCard

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSimCards:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        sim_card = client.sim_cards.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardRetrieveResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Telnyx) -> None:
        sim_card = client.sim_cards.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            include_pin_puk_codes=True,
            include_sim_card_group=True,
        )
        assert_matches_type(SimCardRetrieveResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.sim_cards.with_raw_response.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = response.parse()
        assert_matches_type(SimCardRetrieveResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.sim_cards.with_streaming_response.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = response.parse()
            assert_matches_type(SimCardRetrieveResponse, sim_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_cards.with_raw_response.retrieve(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        sim_card = client.sim_cards.update(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardUpdateResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        sim_card = client.sim_cards.update(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            authorized_imeis=["106516771852751", "534051870479563", "508821468377961"],
            data_limit={
                "amount": "2048.1",
                "unit": "MB",
            },
            sim_card_group_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            status={},
            tags=["personal", "customers", "active-customers"],
        )
        assert_matches_type(SimCardUpdateResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.sim_cards.with_raw_response.update(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = response.parse()
        assert_matches_type(SimCardUpdateResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.sim_cards.with_streaming_response.update(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = response.parse()
            assert_matches_type(SimCardUpdateResponse, sim_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sim_card_id` but received ''"):
            client.sim_cards.with_raw_response.update(
                sim_card_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        sim_card = client.sim_cards.list()
        assert_matches_type(SyncDefaultFlatPagination[SimpleSimCard], sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        sim_card = client.sim_cards.list(
            filter={
                "iccid": "89310410106543789301",
                "msisdn": "+13109976224",
                "status": ["enabled"],
                "tags": ["personal", "customers", "active-customers"],
            },
            filter_sim_card_group_id="47a1c2b0-cc7b-4ab1-bb98-b33fb0fc61b9",
            include_sim_card_group=True,
            page_number=0,
            page_size=0,
            sort="current_billing_period_consumed_data.amount",
        )
        assert_matches_type(SyncDefaultFlatPagination[SimpleSimCard], sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.sim_cards.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[SimpleSimCard], sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.sim_cards.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[SimpleSimCard], sim_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        sim_card = client.sim_cards.delete(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardDeleteResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete_with_all_params(self, client: Telnyx) -> None:
        sim_card = client.sim_cards.delete(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            report_lost=True,
        )
        assert_matches_type(SimCardDeleteResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.sim_cards.with_raw_response.delete(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = response.parse()
        assert_matches_type(SimCardDeleteResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.sim_cards.with_streaming_response.delete(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = response.parse()
            assert_matches_type(SimCardDeleteResponse, sim_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_cards.with_raw_response.delete(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_activation_code(self, client: Telnyx) -> None:
        sim_card = client.sim_cards.get_activation_code(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardGetActivationCodeResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_activation_code(self, client: Telnyx) -> None:
        response = client.sim_cards.with_raw_response.get_activation_code(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = response.parse()
        assert_matches_type(SimCardGetActivationCodeResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_activation_code(self, client: Telnyx) -> None:
        with client.sim_cards.with_streaming_response.get_activation_code(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = response.parse()
            assert_matches_type(SimCardGetActivationCodeResponse, sim_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_activation_code(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_cards.with_raw_response.get_activation_code(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_device_details(self, client: Telnyx) -> None:
        sim_card = client.sim_cards.get_device_details(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardGetDeviceDetailsResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_device_details(self, client: Telnyx) -> None:
        response = client.sim_cards.with_raw_response.get_device_details(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = response.parse()
        assert_matches_type(SimCardGetDeviceDetailsResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_device_details(self, client: Telnyx) -> None:
        with client.sim_cards.with_streaming_response.get_device_details(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = response.parse()
            assert_matches_type(SimCardGetDeviceDetailsResponse, sim_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_device_details(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_cards.with_raw_response.get_device_details(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_public_ip(self, client: Telnyx) -> None:
        sim_card = client.sim_cards.get_public_ip(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardGetPublicIPResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_public_ip(self, client: Telnyx) -> None:
        response = client.sim_cards.with_raw_response.get_public_ip(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = response.parse()
        assert_matches_type(SimCardGetPublicIPResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_public_ip(self, client: Telnyx) -> None:
        with client.sim_cards.with_streaming_response.get_public_ip(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = response.parse()
            assert_matches_type(SimCardGetPublicIPResponse, sim_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_public_ip(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_cards.with_raw_response.get_public_ip(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_wireless_connectivity_logs(self, client: Telnyx) -> None:
        sim_card = client.sim_cards.list_wireless_connectivity_logs(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(
            SyncDefaultFlatPagination[SimCardListWirelessConnectivityLogsResponse], sim_card, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_wireless_connectivity_logs_with_all_params(self, client: Telnyx) -> None:
        sim_card = client.sim_cards.list_wireless_connectivity_logs(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            page_number=1,
            page_size=1,
        )
        assert_matches_type(
            SyncDefaultFlatPagination[SimCardListWirelessConnectivityLogsResponse], sim_card, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_wireless_connectivity_logs(self, client: Telnyx) -> None:
        response = client.sim_cards.with_raw_response.list_wireless_connectivity_logs(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = response.parse()
        assert_matches_type(
            SyncDefaultFlatPagination[SimCardListWirelessConnectivityLogsResponse], sim_card, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_wireless_connectivity_logs(self, client: Telnyx) -> None:
        with client.sim_cards.with_streaming_response.list_wireless_connectivity_logs(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = response.parse()
            assert_matches_type(
                SyncDefaultFlatPagination[SimCardListWirelessConnectivityLogsResponse], sim_card, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list_wireless_connectivity_logs(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_cards.with_raw_response.list_wireless_connectivity_logs(
                id="",
            )


class TestAsyncSimCards:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        sim_card = await async_client.sim_cards.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardRetrieveResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sim_card = await async_client.sim_cards.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            include_pin_puk_codes=True,
            include_sim_card_group=True,
        )
        assert_matches_type(SimCardRetrieveResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_cards.with_raw_response.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = await response.parse()
        assert_matches_type(SimCardRetrieveResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_cards.with_streaming_response.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = await response.parse()
            assert_matches_type(SimCardRetrieveResponse, sim_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_cards.with_raw_response.retrieve(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        sim_card = await async_client.sim_cards.update(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardUpdateResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sim_card = await async_client.sim_cards.update(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            authorized_imeis=["106516771852751", "534051870479563", "508821468377961"],
            data_limit={
                "amount": "2048.1",
                "unit": "MB",
            },
            sim_card_group_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            status={},
            tags=["personal", "customers", "active-customers"],
        )
        assert_matches_type(SimCardUpdateResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_cards.with_raw_response.update(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = await response.parse()
        assert_matches_type(SimCardUpdateResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_cards.with_streaming_response.update(
            sim_card_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = await response.parse()
            assert_matches_type(SimCardUpdateResponse, sim_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sim_card_id` but received ''"):
            await async_client.sim_cards.with_raw_response.update(
                sim_card_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        sim_card = await async_client.sim_cards.list()
        assert_matches_type(AsyncDefaultFlatPagination[SimpleSimCard], sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sim_card = await async_client.sim_cards.list(
            filter={
                "iccid": "89310410106543789301",
                "msisdn": "+13109976224",
                "status": ["enabled"],
                "tags": ["personal", "customers", "active-customers"],
            },
            filter_sim_card_group_id="47a1c2b0-cc7b-4ab1-bb98-b33fb0fc61b9",
            include_sim_card_group=True,
            page_number=0,
            page_size=0,
            sort="current_billing_period_consumed_data.amount",
        )
        assert_matches_type(AsyncDefaultFlatPagination[SimpleSimCard], sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_cards.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[SimpleSimCard], sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_cards.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[SimpleSimCard], sim_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        sim_card = await async_client.sim_cards.delete(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardDeleteResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sim_card = await async_client.sim_cards.delete(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            report_lost=True,
        )
        assert_matches_type(SimCardDeleteResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_cards.with_raw_response.delete(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = await response.parse()
        assert_matches_type(SimCardDeleteResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_cards.with_streaming_response.delete(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = await response.parse()
            assert_matches_type(SimCardDeleteResponse, sim_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_cards.with_raw_response.delete(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_activation_code(self, async_client: AsyncTelnyx) -> None:
        sim_card = await async_client.sim_cards.get_activation_code(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardGetActivationCodeResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_activation_code(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_cards.with_raw_response.get_activation_code(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = await response.parse()
        assert_matches_type(SimCardGetActivationCodeResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_activation_code(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_cards.with_streaming_response.get_activation_code(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = await response.parse()
            assert_matches_type(SimCardGetActivationCodeResponse, sim_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_activation_code(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_cards.with_raw_response.get_activation_code(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_device_details(self, async_client: AsyncTelnyx) -> None:
        sim_card = await async_client.sim_cards.get_device_details(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardGetDeviceDetailsResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_device_details(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_cards.with_raw_response.get_device_details(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = await response.parse()
        assert_matches_type(SimCardGetDeviceDetailsResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_device_details(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_cards.with_streaming_response.get_device_details(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = await response.parse()
            assert_matches_type(SimCardGetDeviceDetailsResponse, sim_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_device_details(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_cards.with_raw_response.get_device_details(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_public_ip(self, async_client: AsyncTelnyx) -> None:
        sim_card = await async_client.sim_cards.get_public_ip(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardGetPublicIPResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_public_ip(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_cards.with_raw_response.get_public_ip(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = await response.parse()
        assert_matches_type(SimCardGetPublicIPResponse, sim_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_public_ip(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_cards.with_streaming_response.get_public_ip(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = await response.parse()
            assert_matches_type(SimCardGetPublicIPResponse, sim_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_public_ip(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_cards.with_raw_response.get_public_ip(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_wireless_connectivity_logs(self, async_client: AsyncTelnyx) -> None:
        sim_card = await async_client.sim_cards.list_wireless_connectivity_logs(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(
            AsyncDefaultFlatPagination[SimCardListWirelessConnectivityLogsResponse], sim_card, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_wireless_connectivity_logs_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sim_card = await async_client.sim_cards.list_wireless_connectivity_logs(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            page_number=1,
            page_size=1,
        )
        assert_matches_type(
            AsyncDefaultFlatPagination[SimCardListWirelessConnectivityLogsResponse], sim_card, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_wireless_connectivity_logs(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_cards.with_raw_response.list_wireless_connectivity_logs(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card = await response.parse()
        assert_matches_type(
            AsyncDefaultFlatPagination[SimCardListWirelessConnectivityLogsResponse], sim_card, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_wireless_connectivity_logs(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_cards.with_streaming_response.list_wireless_connectivity_logs(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[SimCardListWirelessConnectivityLogsResponse], sim_card, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list_wireless_connectivity_logs(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_cards.with_raw_response.list_wireless_connectivity_logs(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["UsecaseGetCostResponse"]


class UsecaseGetCostResponse(BaseModel):
    campaign_usecase: str = FieldInfo(alias="campaignUsecase")

    description: str

    monthly_cost: str = FieldInfo(alias="monthlyCost")

    up_front_cost: str = FieldInfo(alias="upFrontCost")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["Conference", "EndedBy"]


class EndedBy(BaseModel):
    """IDs related to who ended the conference.

    It is expected for them to all be there or all be null
    """

    call_control_id: Optional[str] = None
    """Call Control ID which ended the conference"""

    call_session_id: Optional[str] = None
    """Call Session ID which ended the conference"""


class Conference(BaseModel):
    id: str
    """Uniquely identifies the conference"""

    created_at: str
    """ISO 8601 formatted date of when the conference was created"""

    expires_at: str
    """ISO 8601 formatted date of when the conference will expire"""

    name: str
    """Name of the conference"""

    record_type: Literal["conference"]

    connection_id: Optional[str] = None
    """Identifies the connection associated with the conference"""

    end_reason: Optional[Literal["all_left", "ended_via_api", "host_left", "time_exceeded"]] = None
    """Reason why the conference ended"""

    ended_by: Optional[EndedBy] = None
    """IDs related to who ended the conference.

    It is expected for them to all be there or all be null
    """

    region: Optional[str] = None
    """Region where the conference is hosted"""

    status: Optional[Literal["init", "in_progress", "completed"]] = None
    """Status of the conference"""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date of when the conference was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_recording_transcription_saved import CallRecordingTranscriptionSaved

__all__ = ["CallRecordingTranscriptionSavedWebhookEvent"]


class CallRecordingTranscriptionSavedWebhookEvent(BaseModel):
    data: Optional[CallRecordingTranscriptionSaved] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["VerifiedNumber"]


class VerifiedNumber(BaseModel):
    phone_number: Optional[str] = None

    record_type: Optional[Literal["verified_number"]] = None
    """The possible verified numbers record types."""

    verified_at: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["EventListResponse"]


class EventListResponse(BaseModel):
    event_id: str

    run_id: str

    summary: str

    timestamp: datetime

    type: Literal[
        "status_change",
        "step_started",
        "step_completed",
        "step_failed",
        "tool_call",
        "tool_result",
        "message",
        "error",
        "custom",
    ]

    agent_id: Optional[str] = None

    idempotency_key: Optional[str] = None

    payload: Optional[Dict[str, object]] = None

    step_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["AssociatedPhoneNumberListParams", "Filter", "Sort"]


class AssociatedPhoneNumberListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[phone_number], filter[action]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]

    sort: Sort
    """Consolidated sort parameter (deepObject style). Originally: sort[value]"""


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[phone_number], filter[action]
    """

    action: Literal["keep", "disconnect"]
    """Filter results by action type"""

    phone_number: str
    """Filter results by a phone number. It should be in E.164 format."""


class Sort(TypedDict, total=False):
    """Consolidated sort parameter (deepObject style). Originally: sort[value]"""

    value: Literal["-created_at", "created_at"]
    """Specifies the sort order for results.

    If not given, results are sorted by created_at in descending order
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import inventory_coverage_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.inventory_coverage_list_response import InventoryCoverageListResponse

__all__ = ["InventoryCoverageResource", "AsyncInventoryCoverageResource"]


class InventoryCoverageResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InventoryCoverageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return InventoryCoverageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InventoryCoverageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return InventoryCoverageResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        filter: inventory_coverage_list_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InventoryCoverageListResponse:
        """Creates an inventory coverage request.

        If locality, npa or
        national_destination_code is used in groupBy, and no region or locality filters
        are used, the whole paginated set is returned.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[npa],
              filter[nxx], filter[administrative_area], filter[phone_number_type],
              filter[country_code], filter[count], filter[features], filter[groupBy]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/inventory_coverage",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"filter": filter}, inventory_coverage_list_params.InventoryCoverageListParams),
            ),
            cast_to=InventoryCoverageListResponse,
        )


class AsyncInventoryCoverageResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInventoryCoverageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInventoryCoverageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInventoryCoverageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncInventoryCoverageResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        filter: inventory_coverage_list_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InventoryCoverageListResponse:
        """Creates an inventory coverage request.

        If locality, npa or
        national_destination_code is used in groupBy, and no region or locality filters
        are used, the whole paginated set is returned.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[npa],
              filter[nxx], filter[administrative_area], filter[phone_number_type],
              filter[country_code], filter[count], filter[features], filter[groupBy]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/inventory_coverage",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"filter": filter}, inventory_coverage_list_params.InventoryCoverageListParams
                ),
            ),
            cast_to=InventoryCoverageListResponse,
        )


class InventoryCoverageResourceWithRawResponse:
    def __init__(self, inventory_coverage: InventoryCoverageResource) -> None:
        self._inventory_coverage = inventory_coverage

        self.list = to_raw_response_wrapper(
            inventory_coverage.list,
        )


class AsyncInventoryCoverageResourceWithRawResponse:
    def __init__(self, inventory_coverage: AsyncInventoryCoverageResource) -> None:
        self._inventory_coverage = inventory_coverage

        self.list = async_to_raw_response_wrapper(
            inventory_coverage.list,
        )


class InventoryCoverageResourceWithStreamingResponse:
    def __init__(self, inventory_coverage: InventoryCoverageResource) -> None:
        self._inventory_coverage = inventory_coverage

        self.list = to_streamed_response_wrapper(
            inventory_coverage.list,
        )


class AsyncInventoryCoverageResourceWithStreamingResponse:
    def __init__(self, inventory_coverage: AsyncInventoryCoverageResource) -> None:
        self._inventory_coverage = inventory_coverage

        self.list = async_to_streamed_response_wrapper(
            inventory_coverage.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["MessagingListParams"]


class MessagingListParams(TypedDict, total=False):
    page: int
    """Page number"""

    per_page: int
    """Size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .notification_setting import NotificationSetting

__all__ = ["NotificationSettingDeleteResponse"]


class NotificationSettingDeleteResponse(BaseModel):
    data: Optional[NotificationSetting] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import dynamic_emergency_endpoint_list_params, dynamic_emergency_endpoint_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.dynamic_emergency_endpoint import DynamicEmergencyEndpoint
from ..types.dynamic_emergency_endpoint_create_response import DynamicEmergencyEndpointCreateResponse
from ..types.dynamic_emergency_endpoint_delete_response import DynamicEmergencyEndpointDeleteResponse
from ..types.dynamic_emergency_endpoint_retrieve_response import DynamicEmergencyEndpointRetrieveResponse

__all__ = ["DynamicEmergencyEndpointsResource", "AsyncDynamicEmergencyEndpointsResource"]


class DynamicEmergencyEndpointsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DynamicEmergencyEndpointsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return DynamicEmergencyEndpointsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DynamicEmergencyEndpointsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return DynamicEmergencyEndpointsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        callback_number: str,
        caller_name: str,
        dynamic_emergency_address_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicEmergencyEndpointCreateResponse:
        """
        Creates a dynamic emergency endpoints.

        Args:
          dynamic_emergency_address_id: An id of a currently active dynamic emergency location.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/dynamic_emergency_endpoints",
            body=maybe_transform(
                {
                    "callback_number": callback_number,
                    "caller_name": caller_name,
                    "dynamic_emergency_address_id": dynamic_emergency_address_id,
                },
                dynamic_emergency_endpoint_create_params.DynamicEmergencyEndpointCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicEmergencyEndpointCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicEmergencyEndpointRetrieveResponse:
        """
        Returns the dynamic emergency endpoint based on the ID provided

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/dynamic_emergency_endpoints/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicEmergencyEndpointRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: dynamic_emergency_endpoint_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[DynamicEmergencyEndpoint]:
        """
        Returns the dynamic emergency endpoints according to filters

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[status],
              filter[country_code]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/dynamic_emergency_endpoints",
            page=SyncDefaultFlatPagination[DynamicEmergencyEndpoint],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    dynamic_emergency_endpoint_list_params.DynamicEmergencyEndpointListParams,
                ),
            ),
            model=DynamicEmergencyEndpoint,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicEmergencyEndpointDeleteResponse:
        """
        Deletes the dynamic emergency endpoint based on the ID provided

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/dynamic_emergency_endpoints/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicEmergencyEndpointDeleteResponse,
        )


class AsyncDynamicEmergencyEndpointsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDynamicEmergencyEndpointsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDynamicEmergencyEndpointsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDynamicEmergencyEndpointsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncDynamicEmergencyEndpointsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        callback_number: str,
        caller_name: str,
        dynamic_emergency_address_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicEmergencyEndpointCreateResponse:
        """
        Creates a dynamic emergency endpoints.

        Args:
          dynamic_emergency_address_id: An id of a currently active dynamic emergency location.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/dynamic_emergency_endpoints",
            body=await async_maybe_transform(
                {
                    "callback_number": callback_number,
                    "caller_name": caller_name,
                    "dynamic_emergency_address_id": dynamic_emergency_address_id,
                },
                dynamic_emergency_endpoint_create_params.DynamicEmergencyEndpointCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicEmergencyEndpointCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicEmergencyEndpointRetrieveResponse:
        """
        Returns the dynamic emergency endpoint based on the ID provided

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/dynamic_emergency_endpoints/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicEmergencyEndpointRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: dynamic_emergency_endpoint_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DynamicEmergencyEndpoint, AsyncDefaultFlatPagination[DynamicEmergencyEndpoint]]:
        """
        Returns the dynamic emergency endpoints according to filters

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[status],
              filter[country_code]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/dynamic_emergency_endpoints",
            page=AsyncDefaultFlatPagination[DynamicEmergencyEndpoint],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    dynamic_emergency_endpoint_list_params.DynamicEmergencyEndpointListParams,
                ),
            ),
            model=DynamicEmergencyEndpoint,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DynamicEmergencyEndpointDeleteResponse:
        """
        Deletes the dynamic emergency endpoint based on the ID provided

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/dynamic_emergency_endpoints/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DynamicEmergencyEndpointDeleteResponse,
        )


class DynamicEmergencyEndpointsResourceWithRawResponse:
    def __init__(self, dynamic_emergency_endpoints: DynamicEmergencyEndpointsResource) -> None:
        self._dynamic_emergency_endpoints = dynamic_emergency_endpoints

        self.create = to_raw_response_wrapper(
            dynamic_emergency_endpoints.create,
        )
        self.retrieve = to_raw_response_wrapper(
            dynamic_emergency_endpoints.retrieve,
        )
        self.list = to_raw_response_wrapper(
            dynamic_emergency_endpoints.list,
        )
        self.delete = to_raw_response_wrapper(
            dynamic_emergency_endpoints.delete,
        )


class AsyncDynamicEmergencyEndpointsResourceWithRawResponse:
    def __init__(self, dynamic_emergency_endpoints: AsyncDynamicEmergencyEndpointsResource) -> None:
        self._dynamic_emergency_endpoints = dynamic_emergency_endpoints

        self.create = async_to_raw_response_wrapper(
            dynamic_emergency_endpoints.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            dynamic_emergency_endpoints.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            dynamic_emergency_endpoints.list,
        )
        self.delete = async_to_raw_response_wrapper(
            dynamic_emergency_endpoints.delete,
        )


class DynamicEmergencyEndpointsResourceWithStreamingResponse:
    def __init__(self, dynamic_emergency_endpoints: DynamicEmergencyEndpointsResource) -> None:
        self._dynamic_emergency_endpoints = dynamic_emergency_endpoints

        self.create = to_streamed_response_wrapper(
            dynamic_emergency_endpoints.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            dynamic_emergency_endpoints.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            dynamic_emergency_endpoints.list,
        )
        self.delete = to_streamed_response_wrapper(
            dynamic_emergency_endpoints.delete,
        )


class AsyncDynamicEmergencyEndpointsResourceWithStreamingResponse:
    def __init__(self, dynamic_emergency_endpoints: AsyncDynamicEmergencyEndpointsResource) -> None:
        self._dynamic_emergency_endpoints = dynamic_emergency_endpoints

        self.create = async_to_streamed_response_wrapper(
            dynamic_emergency_endpoints.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            dynamic_emergency_endpoints.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            dynamic_emergency_endpoints.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            dynamic_emergency_endpoints.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncPerPagePaginationV2, AsyncPerPagePaginationV2
from ...._base_client import AsyncPaginator, make_request_options
from .external_vetting import (
    ExternalVettingResource,
    AsyncExternalVettingResource,
    ExternalVettingResourceWithRawResponse,
    AsyncExternalVettingResourceWithRawResponse,
    ExternalVettingResourceWithStreamingResponse,
    AsyncExternalVettingResourceWithStreamingResponse,
)
from ....types.messaging_10dlc import (
    Vertical,
    EntityType,
    StockExchange,
    AltBusinessIDType,
    BrandIdentityStatus,
    brand_list_params,
    brand_create_params,
    brand_update_params,
    brand_verify_sms_otp_params,
    brand_trigger_sms_otp_params,
)
from ....types.messaging_10dlc.vertical import Vertical
from ....types.messaging_10dlc.entity_type import EntityType
from ....types.messaging_10dlc.telnyx_brand import TelnyxBrand
from ....types.messaging_10dlc.stock_exchange import StockExchange
from ....types.messaging_10dlc.brand_list_response import BrandListResponse
from ....types.messaging_10dlc.alt_business_id_type import AltBusinessIDType
from ....types.messaging_10dlc.brand_identity_status import BrandIdentityStatus
from ....types.messaging_10dlc.brand_retrieve_response import BrandRetrieveResponse
from ....types.messaging_10dlc.brand_get_feedback_response import BrandGetFeedbackResponse
from ....types.messaging_10dlc.brand_trigger_sms_otp_response import BrandTriggerSMSOtpResponse
from ....types.messaging_10dlc.brand_retrieve_sms_otp_status_response import BrandRetrieveSMSOtpStatusResponse

__all__ = ["BrandResource", "AsyncBrandResource"]


class BrandResource(SyncAPIResource):
    @cached_property
    def external_vetting(self) -> ExternalVettingResource:
        return ExternalVettingResource(self._client)

    @cached_property
    def with_raw_response(self) -> BrandResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return BrandResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BrandResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return BrandResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        country: str,
        display_name: str,
        email: str,
        entity_type: EntityType,
        vertical: Vertical,
        business_contact_email: str | Omit = omit,
        city: str | Omit = omit,
        company_name: str | Omit = omit,
        ein: str | Omit = omit,
        first_name: str | Omit = omit,
        ip_address: str | Omit = omit,
        is_reseller: bool | Omit = omit,
        last_name: str | Omit = omit,
        mobile_phone: str | Omit = omit,
        mock: bool | Omit = omit,
        phone: str | Omit = omit,
        postal_code: str | Omit = omit,
        state: str | Omit = omit,
        stock_exchange: StockExchange | Omit = omit,
        stock_symbol: str | Omit = omit,
        street: str | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        website: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxBrand:
        """This endpoint is used to create a new brand.

        A brand is an entity created by The
        Campaign Registry (TCR) that represents an organization or a company. It is this
        entity that TCR created campaigns will be associated with. Each brand creation
        will entail an upfront, non-refundable $4 expense.

        Args:
          country: ISO2 2 characters country code. Example: US - United States

          display_name: Display name, marketing name, or DBA name of the brand.

          email: Valid email address of brand support contact.

          entity_type: Entity type behind the brand. This is the form of business establishment.

          vertical: Vertical or industry segment of the brand or campaign.

          business_contact_email: Business contact email.

              Required if `entityType` is `PUBLIC_PROFIT`. Otherwise, it is recommended to
              either omit this field or set it to `null`.

          city: City name

          company_name: (Required for Non-profit/private/public) Legal company name.

          ein: (Required for Non-profit) Government assigned corporate tax ID. EIN is 9-digits
              in U.S.

          first_name: First name of business contact.

          ip_address: IP address of the browser requesting to create brand identity.

          last_name: Last name of business contact.

          mobile_phone: Valid mobile phone number in e.164 international format.

          mock: Mock brand for testing purposes. Defaults to false.

          phone: Valid phone number in e.164 international format.

          postal_code: Postal codes. Use 5 digit zipcode for United States

          state: State. Must be 2 letters code for United States.

          stock_exchange: (Required for public company) stock exchange.

          stock_symbol: (Required for public company) stock symbol.

          street: Street number and name.

          webhook_failover_url: Webhook failover URL for brand status updates.

          webhook_url: Webhook URL for brand status updates.

          website: Brand website URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/10dlc/brand",
            body=maybe_transform(
                {
                    "country": country,
                    "display_name": display_name,
                    "email": email,
                    "entity_type": entity_type,
                    "vertical": vertical,
                    "business_contact_email": business_contact_email,
                    "city": city,
                    "company_name": company_name,
                    "ein": ein,
                    "first_name": first_name,
                    "ip_address": ip_address,
                    "is_reseller": is_reseller,
                    "last_name": last_name,
                    "mobile_phone": mobile_phone,
                    "mock": mock,
                    "phone": phone,
                    "postal_code": postal_code,
                    "state": state,
                    "stock_exchange": stock_exchange,
                    "stock_symbol": stock_symbol,
                    "street": street,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                    "website": website,
                },
                brand_create_params.BrandCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxBrand,
        )

    def retrieve(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandRetrieveResponse:
        """
        Retrieve a brand by `brandId`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return self._get(
            f"/10dlc/brand/{brand_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandRetrieveResponse,
        )

    def update(
        self,
        brand_id: str,
        *,
        country: str,
        display_name: str,
        email: str,
        entity_type: EntityType,
        vertical: Vertical,
        alt_business_id: str | Omit = omit,
        alt_business_id_type: AltBusinessIDType | Omit = omit,
        business_contact_email: str | Omit = omit,
        city: str | Omit = omit,
        company_name: str | Omit = omit,
        ein: str | Omit = omit,
        first_name: str | Omit = omit,
        identity_status: BrandIdentityStatus | Omit = omit,
        ip_address: str | Omit = omit,
        is_reseller: bool | Omit = omit,
        last_name: str | Omit = omit,
        phone: str | Omit = omit,
        postal_code: str | Omit = omit,
        state: str | Omit = omit,
        stock_exchange: StockExchange | Omit = omit,
        stock_symbol: str | Omit = omit,
        street: str | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        website: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxBrand:
        """
        Update a brand's attributes by `brandId`.

        Args:
          country: ISO2 2 characters country code. Example: US - United States

          display_name: Display or marketing name of the brand.

          email: Valid email address of brand support contact.

          entity_type: Entity type behind the brand. This is the form of business establishment.

          vertical: Vertical or industry segment of the brand or campaign.

          alt_business_id: Alternate business identifier such as DUNS, LEI, or GIIN

          alt_business_id_type: An enumeration.

          business_contact_email: Business contact email.

              Required if `entityType` will be changed to `PUBLIC_PROFIT`. Otherwise, it is
              recommended to either omit this field or set it to `null`.

          city: City name

          company_name: (Required for Non-profit/private/public) Legal company name.

          ein: (Required for Non-profit) Government assigned corporate tax ID. EIN is 9-digits
              in U.S.

          first_name: First name of business contact.

          identity_status: The verification status of an active brand

          ip_address: IP address of the browser requesting to create brand identity.

          last_name: Last name of business contact.

          phone: Valid phone number in e.164 international format.

          postal_code: Postal codes. Use 5 digit zipcode for United States

          state: State. Must be 2 letters code for United States.

          stock_exchange: (Required for public company) stock exchange.

          stock_symbol: (Required for public company) stock symbol.

          street: Street number and name.

          webhook_failover_url: Webhook failover URL for brand status updates.

          webhook_url: Webhook URL for brand status updates.

          website: Brand website URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return self._put(
            f"/10dlc/brand/{brand_id}",
            body=maybe_transform(
                {
                    "country": country,
                    "display_name": display_name,
                    "email": email,
                    "entity_type": entity_type,
                    "vertical": vertical,
                    "alt_business_id": alt_business_id,
                    "alt_business_id_type": alt_business_id_type,
                    "business_contact_email": business_contact_email,
                    "city": city,
                    "company_name": company_name,
                    "ein": ein,
                    "first_name": first_name,
                    "identity_status": identity_status,
                    "ip_address": ip_address,
                    "is_reseller": is_reseller,
                    "last_name": last_name,
                    "phone": phone,
                    "postal_code": postal_code,
                    "state": state,
                    "stock_exchange": stock_exchange,
                    "stock_symbol": stock_symbol,
                    "street": street,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                    "website": website,
                },
                brand_update_params.BrandUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxBrand,
        )

    def list(
        self,
        *,
        brand_id: str | Omit = omit,
        country: str | Omit = omit,
        display_name: str | Omit = omit,
        entity_type: str | Omit = omit,
        page: int | Omit = omit,
        records_per_page: int | Omit = omit,
        sort: Literal[
            "assignedCampaignsCount",
            "-assignedCampaignsCount",
            "brandId",
            "-brandId",
            "createdAt",
            "-createdAt",
            "displayName",
            "-displayName",
            "identityStatus",
            "-identityStatus",
            "status",
            "-status",
            "tcrBrandId",
            "-tcrBrandId",
        ]
        | Omit = omit,
        state: str | Omit = omit,
        tcr_brand_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPerPagePaginationV2[BrandListResponse]:
        """
        This endpoint is used to list all brands associated with your organization.

        Args:
          brand_id: Filter results by the Telnyx Brand id

          records_per_page: number of records per page. maximum of 500

          sort: Specifies the sort order for results. If not given, results are sorted by
              createdAt in descending order.

          tcr_brand_id: Filter results by the TCR Brand id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/10dlc/brand",
            page=SyncPerPagePaginationV2[BrandListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "brand_id": brand_id,
                        "country": country,
                        "display_name": display_name,
                        "entity_type": entity_type,
                        "page": page,
                        "records_per_page": records_per_page,
                        "sort": sort,
                        "state": state,
                        "tcr_brand_id": tcr_brand_id,
                    },
                    brand_list_params.BrandListParams,
                ),
            ),
            model=BrandListResponse,
        )

    def delete(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete Brand.

        This endpoint is used to delete a brand. Note the brand cannot be
        deleted if it contains one or more active campaigns, the campaigns need to be
        inactive and at least 3 months old due to billing purposes.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/10dlc/brand/{brand_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get_feedback(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandGetFeedbackResponse:
        """Get feedback about a brand by ID.

        This endpoint can be used after creating or
        revetting a brand.

        Possible values for `.category[].id`:

        - `TAX_ID` - Data mismatch related to tax id and its associated properties.
        - `STOCK_SYMBOL` - Non public entity registered as a public for profit entity or
          the stock information mismatch.
        - `GOVERNMENT_ENTITY` - Non government entity registered as a government entity.
          Must be a U.S. government entity.
        - `NONPROFIT` - Not a recognized non-profit entity. No IRS tax-exempt status
          found.
        - `OTHERS` - Details of the data misrepresentation if any.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return self._get(
            f"/10dlc/brand/feedback/{brand_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandGetFeedbackResponse,
        )

    def resend_2fa_email(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Resend brand 2FA email

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/10dlc/brand/{brand_id}/2faEmail",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def retrieve_sms_otp_status(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandRetrieveSMSOtpStatusResponse:
        """
        Query the status of an SMS OTP (One-Time Password) for Sole Proprietor brand
        verification using the Brand ID.

        This endpoint allows you to check the delivery and verification status of an OTP
        sent during the Sole Proprietor brand verification process by looking it up with
        the brand ID.

        The response includes delivery status, verification dates, and detailed delivery
        information.

        **Note:** This is an alternative to the `/10dlc/brand/smsOtp/{referenceId}`
        endpoint when you have the Brand ID but not the reference ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return self._get(
            f"/10dlc/brand/{brand_id}/smsOtp",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandRetrieveSMSOtpStatusResponse,
        )

    def revet(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxBrand:
        """This operation allows you to revet the brand.

        However, revetting is allowed once
        after the successful brand registration and thereafter limited to once every 3
        months.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return self._put(
            f"/10dlc/brand/{brand_id}/revet",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxBrand,
        )

    def trigger_sms_otp(
        self,
        brand_id: str,
        *,
        pin_sms: str,
        success_sms: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandTriggerSMSOtpResponse:
        """
        Trigger or re-trigger an SMS OTP (One-Time Password) for Sole Proprietor brand
        verification.

        **Important Notes:**

        - Only allowed for Sole Proprietor (`SOLE_PROPRIETOR`) brands
        - Triggers generation of a one-time password sent to the `mobilePhone` number in
          the brand's profile
        - Campaigns cannot be created until OTP verification is complete
        - US/CA numbers only for real OTPs; mock brands can use non-US/CA numbers for
          testing
        - Returns a `referenceId` that can be used to check OTP status via the GET
          `/10dlc/brand/smsOtp/{referenceId}` endpoint

        **Use Cases:**

        - Initial OTP trigger after Sole Proprietor brand creation
        - Re-triggering OTP if the user didn't receive or needs a new code

        Args:
          pin_sms: SMS message template to send the OTP. Must include `@OTP_PIN@` placeholder which
              will be replaced with the actual PIN

          success_sms: SMS message to send upon successful OTP verification

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return self._post(
            f"/10dlc/brand/{brand_id}/smsOtp",
            body=maybe_transform(
                {
                    "pin_sms": pin_sms,
                    "success_sms": success_sms,
                },
                brand_trigger_sms_otp_params.BrandTriggerSMSOtpParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandTriggerSMSOtpResponse,
        )

    def verify_sms_otp(
        self,
        brand_id: str,
        *,
        otp_pin: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Verify the SMS OTP (One-Time Password) for Sole Proprietor brand verification.

        **Verification Flow:**

        1. User receives OTP via SMS after triggering
        2. User submits the OTP pin through this endpoint
        3. Upon successful verification:
           - A `BRAND_OTP_VERIFIED` webhook event is sent to the CSP
           - The brand's `identityStatus` changes to `VERIFIED`
           - Campaigns can now be created for this brand

        **Error Handling:**

        Provides proper error responses for:

        - Invalid OTP pins
        - Expired OTPs
        - OTP verification failures

        Args:
          otp_pin: The OTP PIN received via SMS

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            f"/10dlc/brand/{brand_id}/smsOtp",
            body=maybe_transform({"otp_pin": otp_pin}, brand_verify_sms_otp_params.BrandVerifySMSOtpParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncBrandResource(AsyncAPIResource):
    @cached_property
    def external_vetting(self) -> AsyncExternalVettingResource:
        return AsyncExternalVettingResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBrandResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBrandResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBrandResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncBrandResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        country: str,
        display_name: str,
        email: str,
        entity_type: EntityType,
        vertical: Vertical,
        business_contact_email: str | Omit = omit,
        city: str | Omit = omit,
        company_name: str | Omit = omit,
        ein: str | Omit = omit,
        first_name: str | Omit = omit,
        ip_address: str | Omit = omit,
        is_reseller: bool | Omit = omit,
        last_name: str | Omit = omit,
        mobile_phone: str | Omit = omit,
        mock: bool | Omit = omit,
        phone: str | Omit = omit,
        postal_code: str | Omit = omit,
        state: str | Omit = omit,
        stock_exchange: StockExchange | Omit = omit,
        stock_symbol: str | Omit = omit,
        street: str | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        website: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxBrand:
        """This endpoint is used to create a new brand.

        A brand is an entity created by The
        Campaign Registry (TCR) that represents an organization or a company. It is this
        entity that TCR created campaigns will be associated with. Each brand creation
        will entail an upfront, non-refundable $4 expense.

        Args:
          country: ISO2 2 characters country code. Example: US - United States

          display_name: Display name, marketing name, or DBA name of the brand.

          email: Valid email address of brand support contact.

          entity_type: Entity type behind the brand. This is the form of business establishment.

          vertical: Vertical or industry segment of the brand or campaign.

          business_contact_email: Business contact email.

              Required if `entityType` is `PUBLIC_PROFIT`. Otherwise, it is recommended to
              either omit this field or set it to `null`.

          city: City name

          company_name: (Required for Non-profit/private/public) Legal company name.

          ein: (Required for Non-profit) Government assigned corporate tax ID. EIN is 9-digits
              in U.S.

          first_name: First name of business contact.

          ip_address: IP address of the browser requesting to create brand identity.

          last_name: Last name of business contact.

          mobile_phone: Valid mobile phone number in e.164 international format.

          mock: Mock brand for testing purposes. Defaults to false.

          phone: Valid phone number in e.164 international format.

          postal_code: Postal codes. Use 5 digit zipcode for United States

          state: State. Must be 2 letters code for United States.

          stock_exchange: (Required for public company) stock exchange.

          stock_symbol: (Required for public company) stock symbol.

          street: Street number and name.

          webhook_failover_url: Webhook failover URL for brand status updates.

          webhook_url: Webhook URL for brand status updates.

          website: Brand website URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/10dlc/brand",
            body=await async_maybe_transform(
                {
                    "country": country,
                    "display_name": display_name,
                    "email": email,
                    "entity_type": entity_type,
                    "vertical": vertical,
                    "business_contact_email": business_contact_email,
                    "city": city,
                    "company_name": company_name,
                    "ein": ein,
                    "first_name": first_name,
                    "ip_address": ip_address,
                    "is_reseller": is_reseller,
                    "last_name": last_name,
                    "mobile_phone": mobile_phone,
                    "mock": mock,
                    "phone": phone,
                    "postal_code": postal_code,
                    "state": state,
                    "stock_exchange": stock_exchange,
                    "stock_symbol": stock_symbol,
                    "street": street,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                    "website": website,
                },
                brand_create_params.BrandCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxBrand,
        )

    async def retrieve(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandRetrieveResponse:
        """
        Retrieve a brand by `brandId`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return await self._get(
            f"/10dlc/brand/{brand_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandRetrieveResponse,
        )

    async def update(
        self,
        brand_id: str,
        *,
        country: str,
        display_name: str,
        email: str,
        entity_type: EntityType,
        vertical: Vertical,
        alt_business_id: str | Omit = omit,
        alt_business_id_type: AltBusinessIDType | Omit = omit,
        business_contact_email: str | Omit = omit,
        city: str | Omit = omit,
        company_name: str | Omit = omit,
        ein: str | Omit = omit,
        first_name: str | Omit = omit,
        identity_status: BrandIdentityStatus | Omit = omit,
        ip_address: str | Omit = omit,
        is_reseller: bool | Omit = omit,
        last_name: str | Omit = omit,
        phone: str | Omit = omit,
        postal_code: str | Omit = omit,
        state: str | Omit = omit,
        stock_exchange: StockExchange | Omit = omit,
        stock_symbol: str | Omit = omit,
        street: str | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        website: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxBrand:
        """
        Update a brand's attributes by `brandId`.

        Args:
          country: ISO2 2 characters country code. Example: US - United States

          display_name: Display or marketing name of the brand.

          email: Valid email address of brand support contact.

          entity_type: Entity type behind the brand. This is the form of business establishment.

          vertical: Vertical or industry segment of the brand or campaign.

          alt_business_id: Alternate business identifier such as DUNS, LEI, or GIIN

          alt_business_id_type: An enumeration.

          business_contact_email: Business contact email.

              Required if `entityType` will be changed to `PUBLIC_PROFIT`. Otherwise, it is
              recommended to either omit this field or set it to `null`.

          city: City name

          company_name: (Required for Non-profit/private/public) Legal company name.

          ein: (Required for Non-profit) Government assigned corporate tax ID. EIN is 9-digits
              in U.S.

          first_name: First name of business contact.

          identity_status: The verification status of an active brand

          ip_address: IP address of the browser requesting to create brand identity.

          last_name: Last name of business contact.

          phone: Valid phone number in e.164 international format.

          postal_code: Postal codes. Use 5 digit zipcode for United States

          state: State. Must be 2 letters code for United States.

          stock_exchange: (Required for public company) stock exchange.

          stock_symbol: (Required for public company) stock symbol.

          street: Street number and name.

          webhook_failover_url: Webhook failover URL for brand status updates.

          webhook_url: Webhook URL for brand status updates.

          website: Brand website URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return await self._put(
            f"/10dlc/brand/{brand_id}",
            body=await async_maybe_transform(
                {
                    "country": country,
                    "display_name": display_name,
                    "email": email,
                    "entity_type": entity_type,
                    "vertical": vertical,
                    "alt_business_id": alt_business_id,
                    "alt_business_id_type": alt_business_id_type,
                    "business_contact_email": business_contact_email,
                    "city": city,
                    "company_name": company_name,
                    "ein": ein,
                    "first_name": first_name,
                    "identity_status": identity_status,
                    "ip_address": ip_address,
                    "is_reseller": is_reseller,
                    "last_name": last_name,
                    "phone": phone,
                    "postal_code": postal_code,
                    "state": state,
                    "stock_exchange": stock_exchange,
                    "stock_symbol": stock_symbol,
                    "street": street,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                    "website": website,
                },
                brand_update_params.BrandUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxBrand,
        )

    def list(
        self,
        *,
        brand_id: str | Omit = omit,
        country: str | Omit = omit,
        display_name: str | Omit = omit,
        entity_type: str | Omit = omit,
        page: int | Omit = omit,
        records_per_page: int | Omit = omit,
        sort: Literal[
            "assignedCampaignsCount",
            "-assignedCampaignsCount",
            "brandId",
            "-brandId",
            "createdAt",
            "-createdAt",
            "displayName",
            "-displayName",
            "identityStatus",
            "-identityStatus",
            "status",
            "-status",
            "tcrBrandId",
            "-tcrBrandId",
        ]
        | Omit = omit,
        state: str | Omit = omit,
        tcr_brand_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[BrandListResponse, AsyncPerPagePaginationV2[BrandListResponse]]:
        """
        This endpoint is used to list all brands associated with your organization.

        Args:
          brand_id: Filter results by the Telnyx Brand id

          records_per_page: number of records per page. maximum of 500

          sort: Specifies the sort order for results. If not given, results are sorted by
              createdAt in descending order.

          tcr_brand_id: Filter results by the TCR Brand id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/10dlc/brand",
            page=AsyncPerPagePaginationV2[BrandListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "brand_id": brand_id,
                        "country": country,
                        "display_name": display_name,
                        "entity_type": entity_type,
                        "page": page,
                        "records_per_page": records_per_page,
                        "sort": sort,
                        "state": state,
                        "tcr_brand_id": tcr_brand_id,
                    },
                    brand_list_params.BrandListParams,
                ),
            ),
            model=BrandListResponse,
        )

    async def delete(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete Brand.

        This endpoint is used to delete a brand. Note the brand cannot be
        deleted if it contains one or more active campaigns, the campaigns need to be
        inactive and at least 3 months old due to billing purposes.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/10dlc/brand/{brand_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get_feedback(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandGetFeedbackResponse:
        """Get feedback about a brand by ID.

        This endpoint can be used after creating or
        revetting a brand.

        Possible values for `.category[].id`:

        - `TAX_ID` - Data mismatch related to tax id and its associated properties.
        - `STOCK_SYMBOL` - Non public entity registered as a public for profit entity or
          the stock information mismatch.
        - `GOVERNMENT_ENTITY` - Non government entity registered as a government entity.
          Must be a U.S. government entity.
        - `NONPROFIT` - Not a recognized non-profit entity. No IRS tax-exempt status
          found.
        - `OTHERS` - Details of the data misrepresentation if any.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return await self._get(
            f"/10dlc/brand/feedback/{brand_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandGetFeedbackResponse,
        )

    async def resend_2fa_email(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Resend brand 2FA email

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/10dlc/brand/{brand_id}/2faEmail",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def retrieve_sms_otp_status(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandRetrieveSMSOtpStatusResponse:
        """
        Query the status of an SMS OTP (One-Time Password) for Sole Proprietor brand
        verification using the Brand ID.

        This endpoint allows you to check the delivery and verification status of an OTP
        sent during the Sole Proprietor brand verification process by looking it up with
        the brand ID.

        The response includes delivery status, verification dates, and detailed delivery
        information.

        **Note:** This is an alternative to the `/10dlc/brand/smsOtp/{referenceId}`
        endpoint when you have the Brand ID but not the reference ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return await self._get(
            f"/10dlc/brand/{brand_id}/smsOtp",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandRetrieveSMSOtpStatusResponse,
        )

    async def revet(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxBrand:
        """This operation allows you to revet the brand.

        However, revetting is allowed once
        after the successful brand registration and thereafter limited to once every 3
        months.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return await self._put(
            f"/10dlc/brand/{brand_id}/revet",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxBrand,
        )

    async def trigger_sms_otp(
        self,
        brand_id: str,
        *,
        pin_sms: str,
        success_sms: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandTriggerSMSOtpResponse:
        """
        Trigger or re-trigger an SMS OTP (One-Time Password) for Sole Proprietor brand
        verification.

        **Important Notes:**

        - Only allowed for Sole Proprietor (`SOLE_PROPRIETOR`) brands
        - Triggers generation of a one-time password sent to the `mobilePhone` number in
          the brand's profile
        - Campaigns cannot be created until OTP verification is complete
        - US/CA numbers only for real OTPs; mock brands can use non-US/CA numbers for
          testing
        - Returns a `referenceId` that can be used to check OTP status via the GET
          `/10dlc/brand/smsOtp/{referenceId}` endpoint

        **Use Cases:**

        - Initial OTP trigger after Sole Proprietor brand creation
        - Re-triggering OTP if the user didn't receive or needs a new code

        Args:
          pin_sms: SMS message template to send the OTP. Must include `@OTP_PIN@` placeholder which
              will be replaced with the actual PIN

          success_sms: SMS message to send upon successful OTP verification

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return await self._post(
            f"/10dlc/brand/{brand_id}/smsOtp",
            body=await async_maybe_transform(
                {
                    "pin_sms": pin_sms,
                    "success_sms": success_sms,
                },
                brand_trigger_sms_otp_params.BrandTriggerSMSOtpParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandTriggerSMSOtpResponse,
        )

    async def verify_sms_otp(
        self,
        brand_id: str,
        *,
        otp_pin: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Verify the SMS OTP (One-Time Password) for Sole Proprietor brand verification.

        **Verification Flow:**

        1. User receives OTP via SMS after triggering
        2. User submits the OTP pin through this endpoint
        3. Upon successful verification:
           - A `BRAND_OTP_VERIFIED` webhook event is sent to the CSP
           - The brand's `identityStatus` changes to `VERIFIED`
           - Campaigns can now be created for this brand

        **Error Handling:**

        Provides proper error responses for:

        - Invalid OTP pins
        - Expired OTPs
        - OTP verification failures

        Args:
          otp_pin: The OTP PIN received via SMS

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            f"/10dlc/brand/{brand_id}/smsOtp",
            body=await async_maybe_transform({"otp_pin": otp_pin}, brand_verify_sms_otp_params.BrandVerifySMSOtpParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class BrandResourceWithRawResponse:
    def __init__(self, brand: BrandResource) -> None:
        self._brand = brand

        self.create = to_raw_response_wrapper(
            brand.create,
        )
        self.retrieve = to_raw_response_wrapper(
            brand.retrieve,
        )
        self.update = to_raw_response_wrapper(
            brand.update,
        )
        self.list = to_raw_response_wrapper(
            brand.list,
        )
        self.delete = to_raw_response_wrapper(
            brand.delete,
        )
        self.get_feedback = to_raw_response_wrapper(
            brand.get_feedback,
        )
        self.resend_2fa_email = to_raw_response_wrapper(
            brand.resend_2fa_email,
        )
        self.retrieve_sms_otp_status = to_raw_response_wrapper(
            brand.retrieve_sms_otp_status,
        )
        self.revet = to_raw_response_wrapper(
            brand.revet,
        )
        self.trigger_sms_otp = to_raw_response_wrapper(
            brand.trigger_sms_otp,
        )
        self.verify_sms_otp = to_raw_response_wrapper(
            brand.verify_sms_otp,
        )

    @cached_property
    def external_vetting(self) -> ExternalVettingResourceWithRawResponse:
        return ExternalVettingResourceWithRawResponse(self._brand.external_vetting)


class AsyncBrandResourceWithRawResponse:
    def __init__(self, brand: AsyncBrandResource) -> None:
        self._brand = brand

        self.create = async_to_raw_response_wrapper(
            brand.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            brand.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            brand.update,
        )
        self.list = async_to_raw_response_wrapper(
            brand.list,
        )
        self.delete = async_to_raw_response_wrapper(
            brand.delete,
        )
        self.get_feedback = async_to_raw_response_wrapper(
            brand.get_feedback,
        )
        self.resend_2fa_email = async_to_raw_response_wrapper(
            brand.resend_2fa_email,
        )
        self.retrieve_sms_otp_status = async_to_raw_response_wrapper(
            brand.retrieve_sms_otp_status,
        )
        self.revet = async_to_raw_response_wrapper(
            brand.revet,
        )
        self.trigger_sms_otp = async_to_raw_response_wrapper(
            brand.trigger_sms_otp,
        )
        self.verify_sms_otp = async_to_raw_response_wrapper(
            brand.verify_sms_otp,
        )

    @cached_property
    def external_vetting(self) -> AsyncExternalVettingResourceWithRawResponse:
        return AsyncExternalVettingResourceWithRawResponse(self._brand.external_vetting)


class BrandResourceWithStreamingResponse:
    def __init__(self, brand: BrandResource) -> None:
        self._brand = brand

        self.create = to_streamed_response_wrapper(
            brand.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            brand.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            brand.update,
        )
        self.list = to_streamed_response_wrapper(
            brand.list,
        )
        self.delete = to_streamed_response_wrapper(
            brand.delete,
        )
        self.get_feedback = to_streamed_response_wrapper(
            brand.get_feedback,
        )
        self.resend_2fa_email = to_streamed_response_wrapper(
            brand.resend_2fa_email,
        )
        self.retrieve_sms_otp_status = to_streamed_response_wrapper(
            brand.retrieve_sms_otp_status,
        )
        self.revet = to_streamed_response_wrapper(
            brand.revet,
        )
        self.trigger_sms_otp = to_streamed_response_wrapper(
            brand.trigger_sms_otp,
        )
        self.verify_sms_otp = to_streamed_response_wrapper(
            brand.verify_sms_otp,
        )

    @cached_property
    def external_vetting(self) -> ExternalVettingResourceWithStreamingResponse:
        return ExternalVettingResourceWithStreamingResponse(self._brand.external_vetting)


class AsyncBrandResourceWithStreamingResponse:
    def __init__(self, brand: AsyncBrandResource) -> None:
        self._brand = brand

        self.create = async_to_streamed_response_wrapper(
            brand.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            brand.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            brand.update,
        )
        self.list = async_to_streamed_response_wrapper(
            brand.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            brand.delete,
        )
        self.get_feedback = async_to_streamed_response_wrapper(
            brand.get_feedback,
        )
        self.resend_2fa_email = async_to_streamed_response_wrapper(
            brand.resend_2fa_email,
        )
        self.retrieve_sms_otp_status = async_to_streamed_response_wrapper(
            brand.retrieve_sms_otp_status,
        )
        self.revet = async_to_streamed_response_wrapper(
            brand.revet,
        )
        self.trigger_sms_otp = async_to_streamed_response_wrapper(
            brand.trigger_sms_otp,
        )
        self.verify_sms_otp = async_to_streamed_response_wrapper(
            brand.verify_sms_otp,
        )

    @cached_property
    def external_vetting(self) -> AsyncExternalVettingResourceWithStreamingResponse:
        return AsyncExternalVettingResourceWithStreamingResponse(self._brand.external_vetting)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Iterable
from typing_extensions import Required, TypedDict

from ..enabled_features import EnabledFeatures
from ..assistant_tool_param import AssistantToolParam
from ..voice_settings_param import VoiceSettingsParam
from ..widget_settings_param import WidgetSettingsParam
from ..insight_settings_param import InsightSettingsParam
from ..privacy_settings_param import PrivacySettingsParam
from ..messaging_settings_param import MessagingSettingsParam
from ..telephony_settings_param import TelephonySettingsParam
from ..transcription_settings_param import TranscriptionSettingsParam

__all__ = ["VersionUpdateParams"]


class VersionUpdateParams(TypedDict, total=False):
    assistant_id: Required[str]

    description: str

    dynamic_variables: Dict[str, object]
    """Map of dynamic variables and their default values"""

    dynamic_variables_webhook_url: str
    """
    If the dynamic_variables_webhook_url is set for the assistant, we will send a
    request at the start of the conversation. See our
    [guide](https://developers.telnyx.com/docs/inference/ai-assistants/dynamic-variables)
    for more information.
    """

    enabled_features: List[EnabledFeatures]

    greeting: str
    """Text that the assistant will use to start the conversation.

    This may be templated with
    [dynamic variables](https://developers.telnyx.com/docs/inference/ai-assistants/dynamic-variables).
    Use an empty string to have the assistant wait for the user to speak first. Use
    the special value `<assistant-speaks-first-with-model-generated-message>` to
    have the assistant generate the greeting based on the system instructions.
    """

    insight_settings: InsightSettingsParam

    instructions: str
    """System instructions for the assistant.

    These may be templated with
    [dynamic variables](https://developers.telnyx.com/docs/inference/ai-assistants/dynamic-variables)
    """

    llm_api_key_ref: str
    """This is only needed when using third-party inference providers.

    The `identifier` for an integration secret
    [/v2/integration_secrets](https://developers.telnyx.com/api-reference/integration-secrets/create-a-secret)
    that refers to your LLM provider's API key. Warning: Free plans are unlikely to
    work with this integration.
    """

    messaging_settings: MessagingSettingsParam

    model: str
    """ID of the model to use.

    You can use the
    [Get models API](https://developers.telnyx.com/api-reference/chat/get-available-models)
    to see all of your available models,
    """

    name: str

    privacy_settings: PrivacySettingsParam

    telephony_settings: TelephonySettingsParam

    tools: Iterable[AssistantToolParam]
    """The tools that the assistant can use.

    These may be templated with
    [dynamic variables](https://developers.telnyx.com/docs/inference/ai-assistants/dynamic-variables)
    """

    transcription: TranscriptionSettingsParam

    voice_settings: VoiceSettingsParam

    widget_settings: WidgetSettingsParam
    """Configuration settings for the assistant's web widget."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["Fqdn"]


class Fqdn(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    connection_id: Optional[str] = None
    """ID of the FQDN connection to which this FQDN is attached."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    dns_record_type: Optional[str] = None
    """The DNS record type for the FQDN.

    For cases where a port is not set, the DNS record type must be 'srv'. For cases
    where a port is set, the DNS record type must be 'a'. If the DNS record type is
    'a' and a port is not specified, 5060 will be used.
    """

    fqdn: Optional[str] = None
    """FQDN represented by this resource."""

    port: Optional[int] = None
    """Port to use when connecting to this FQDN."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AssociatedPhoneNumberCreateParams", "PhoneNumberRange"]


class AssociatedPhoneNumberCreateParams(TypedDict, total=False):
    action: Required[Literal["keep", "disconnect"]]
    """Specifies the action to take with this phone number during partial porting."""

    phone_number_range: Required[PhoneNumberRange]


class PhoneNumberRange(TypedDict, total=False):
    end_at: str
    """Specifies the end of the phone number range for this associated phone number."""

    start_at: str
    """Specifies the start of the phone number range for this associated phone number."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .rcs import (
    RcsResource,
    AsyncRcsResource,
    RcsResourceWithRawResponse,
    AsyncRcsResourceWithRawResponse,
    RcsResourceWithStreamingResponse,
    AsyncRcsResourceWithStreamingResponse,
)
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)

__all__ = [
    "RcsResource",
    "AsyncRcsResource",
    "RcsResourceWithRawResponse",
    "AsyncRcsResourceWithRawResponse",
    "RcsResourceWithStreamingResponse",
    "AsyncRcsResourceWithStreamingResponse",
    "MessagesResource",
    "AsyncMessagesResource",
    "MessagesResourceWithRawResponse",
    "AsyncMessagesResourceWithRawResponse",
    "MessagesResourceWithStreamingResponse",
    "AsyncMessagesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .tn_upload_entry import TnUploadEntry

__all__ = ["Upload"]


class Upload(BaseModel):
    available_usages: Optional[List[Literal["calling_user_assignment", "first_party_app_assignment"]]] = None

    error_code: Optional[str] = None
    """
    A code returned by Microsoft Teams if there is an error with the upload process.
    """

    error_message: Optional[str] = None
    """A message set if there is an error with the upload process."""

    location_id: Optional[str] = None

    status: Optional[Literal["pending_upload", "pending", "in_progress", "partial_success", "success", "error"]] = None
    """Represents the status of the upload on Microsoft Teams."""

    tenant_id: Optional[str] = None

    ticket_id: Optional[str] = None
    """Uniquely identifies the resource."""

    tn_upload_entries: Optional[List[TnUploadEntry]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .action_kick_params import ActionKickParams as ActionKickParams
from .action_mute_params import ActionMuteParams as ActionMuteParams
from .action_end_response import ActionEndResponse as ActionEndResponse
from .action_kick_response import ActionKickResponse as ActionKickResponse
from .action_mute_response import ActionMuteResponse as ActionMuteResponse
from .action_unmute_params import ActionUnmuteParams as ActionUnmuteParams
from .action_unmute_response import ActionUnmuteResponse as ActionUnmuteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    Room,
    RoomCreateResponse,
    RoomUpdateResponse,
    RoomRetrieveResponse,
)
from telnyx._utils import parse_date
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRooms:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        room = client.rooms.create()
        assert_matches_type(RoomCreateResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        room = client.rooms.create(
            enable_recording=True,
            max_participants=10,
            unique_name="My room",
            webhook_event_failover_url="https://failover.example.com",
            webhook_event_url="https://example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(RoomCreateResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.rooms.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room = response.parse()
        assert_matches_type(RoomCreateResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.rooms.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room = response.parse()
            assert_matches_type(RoomCreateResponse, room, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        room = client.rooms.retrieve(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert_matches_type(RoomRetrieveResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Telnyx) -> None:
        room = client.rooms.retrieve(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
            include_sessions=True,
        )
        assert_matches_type(RoomRetrieveResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.rooms.with_raw_response.retrieve(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room = response.parse()
        assert_matches_type(RoomRetrieveResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.rooms.with_streaming_response.retrieve(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room = response.parse()
            assert_matches_type(RoomRetrieveResponse, room, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_id` but received ''"):
            client.rooms.with_raw_response.retrieve(
                room_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        room = client.rooms.update(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert_matches_type(RoomUpdateResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        room = client.rooms.update(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
            enable_recording=True,
            max_participants=10,
            unique_name="My room",
            webhook_event_failover_url="https://failover.example.com",
            webhook_event_url="https://example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(RoomUpdateResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.rooms.with_raw_response.update(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room = response.parse()
        assert_matches_type(RoomUpdateResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.rooms.with_streaming_response.update(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room = response.parse()
            assert_matches_type(RoomUpdateResponse, room, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_id` but received ''"):
            client.rooms.with_raw_response.update(
                room_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        room = client.rooms.list()
        assert_matches_type(SyncDefaultFlatPagination[Room], room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        room = client.rooms.list(
            filter={
                "date_created_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_updated_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "unique_name": "my_video_room",
            },
            include_sessions=True,
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[Room], room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.rooms.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[Room], room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.rooms.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[Room], room, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        room = client.rooms.delete(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert room is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.rooms.with_raw_response.delete(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room = response.parse()
        assert room is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.rooms.with_streaming_response.delete(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room = response.parse()
            assert room is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_id` but received ''"):
            client.rooms.with_raw_response.delete(
                "",
            )


class TestAsyncRooms:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        room = await async_client.rooms.create()
        assert_matches_type(RoomCreateResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        room = await async_client.rooms.create(
            enable_recording=True,
            max_participants=10,
            unique_name="My room",
            webhook_event_failover_url="https://failover.example.com",
            webhook_event_url="https://example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(RoomCreateResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.rooms.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room = await response.parse()
        assert_matches_type(RoomCreateResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.rooms.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room = await response.parse()
            assert_matches_type(RoomCreateResponse, room, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        room = await async_client.rooms.retrieve(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert_matches_type(RoomRetrieveResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncTelnyx) -> None:
        room = await async_client.rooms.retrieve(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
            include_sessions=True,
        )
        assert_matches_type(RoomRetrieveResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.rooms.with_raw_response.retrieve(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room = await response.parse()
        assert_matches_type(RoomRetrieveResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.rooms.with_streaming_response.retrieve(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room = await response.parse()
            assert_matches_type(RoomRetrieveResponse, room, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_id` but received ''"):
            await async_client.rooms.with_raw_response.retrieve(
                room_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        room = await async_client.rooms.update(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert_matches_type(RoomUpdateResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        room = await async_client.rooms.update(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
            enable_recording=True,
            max_participants=10,
            unique_name="My room",
            webhook_event_failover_url="https://failover.example.com",
            webhook_event_url="https://example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(RoomUpdateResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.rooms.with_raw_response.update(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room = await response.parse()
        assert_matches_type(RoomUpdateResponse, room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.rooms.with_streaming_response.update(
            room_id="0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room = await response.parse()
            assert_matches_type(RoomUpdateResponse, room, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_id` but received ''"):
            await async_client.rooms.with_raw_response.update(
                room_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        room = await async_client.rooms.list()
        assert_matches_type(AsyncDefaultFlatPagination[Room], room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        room = await async_client.rooms.list(
            filter={
                "date_created_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_updated_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "unique_name": "my_video_room",
            },
            include_sessions=True,
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[Room], room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.rooms.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[Room], room, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.rooms.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[Room], room, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        room = await async_client.rooms.delete(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert room is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.rooms.with_raw_response.delete(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room = await response.parse()
        assert room is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.rooms.with_streaming_response.delete(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room = await response.parse()
            assert room is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_id` but received ''"):
            await async_client.rooms.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .insight_template import InsightTemplate as InsightTemplate
from .insight_list_params import InsightListParams as InsightListParams
from .insight_create_params import InsightCreateParams as InsightCreateParams
from .insight_update_params import InsightUpdateParams as InsightUpdateParams
from .message_list_response import MessageListResponse as MessageListResponse
from .insight_template_group import InsightTemplateGroup as InsightTemplateGroup
from .insight_template_detail import InsightTemplateDetail as InsightTemplateDetail
from .insight_group_update_params import InsightGroupUpdateParams as InsightGroupUpdateParams
from .insight_template_group_detail import InsightTemplateGroupDetail as InsightTemplateGroupDetail
from .insight_group_insight_groups_params import InsightGroupInsightGroupsParams as InsightGroupInsightGroupsParams
from .insight_group_retrieve_insight_groups_params import (
    InsightGroupRetrieveInsightGroupsParams as InsightGroupRetrieveInsightGroupsParams,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["MessagingNumbersBulkUpdateRetrieveResponse", "Data"]


class Data(BaseModel):
    failed: Optional[List[str]] = None
    """Phone numbers that failed to update."""

    order_id: Optional[str] = None
    """Order ID to verify bulk update status."""

    pending: Optional[List[str]] = None
    """Phone numbers pending to be updated."""

    record_type: Optional[Literal["messaging_numbers_bulk_update"]] = None
    """Identifies the type of the resource."""

    success: Optional[List[str]] = None
    """Phoned numbers updated successfully."""


class MessagingNumbersBulkUpdateRetrieveResponse(BaseModel):
    data: Optional[Data] = None
